/*
 *  Copyright (C) 2009  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <linux/module.h>
/* Needed for procfs register functions */
#include <linux/proc_fs.h>
/* Needed for using the seq_file infrastructure */
#include <linux/seq_file.h>

MODULE_AUTHOR("Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>");
MODULE_LICENSE("GPL");

/* write our message to the sequential file */
static int hello_show(struct seq_file *file, void *data)
{
	seq_puts(file, "Hello, world!\n");
	return 0;
}

/* use the single sequential file */
static int hello_procfs_open(struct inode *inode, struct file *file)
{
	return single_open(file, hello_show, NULL);
}

/* our file operations use the seq_file stuff to make it an easy task */
static const struct file_operations hello_procfs_fops =
{
	.open = hello_procfs_open,
	.read = seq_read,
	.llseek = seq_lseek,
	.release = seq_release,
};

static int hello_procfs_init(void)
{
	/* create the proc entry and set its fops: there is a potential race
	 * here */
	struct proc_dir_entry *entry;
	entry = proc_create("hellop", 0666, NULL, &hello_procfs_fops);
	return 0;
}

static void hello_procfs_exit(void)
{
	/* remove our proc entry */
	remove_proc_entry("hellop", NULL);
}

module_init(hello_procfs_init);
module_exit(hello_procfs_exit);
